@@ -4,6 +4,7 @@ import json
 import os
 import sys
 from importlib.abc import InspectLoader
+from pathlib import Path
 from types import ModuleType
 from typing import Any, Dict, Iterable, List, Mapping, Optional, TextIO, Union, cast
 
@@ -25,12 +26,13 @@ else:  # pragma: no cover
 from ._version import __version__, __version_tuple__  # noqa: F401
 from .configuration import Configuration
 from .configuration_set import ConfigurationSet
-from .helpers import InterpolateEnumType, InterpolateType
+from .helpers import InterpolateEnumType, InterpolateType, parse_env_line
 
 
 def config(
     *configs: Iterable,
     prefix: str = "",
+    strip_prefix: bool = True,
     separator: Optional[str] = None,
     remove_level: int = 1,
     lowercase_keys: bool = False,
@@ -43,6 +45,7 @@ def config(
     Params:
        configs: iterable of configurations
        prefix: prefix to filter environment variables with
+       strip_prefix: whether to strip the prefix
        remove_level: how many levels to remove from the resulting config
        lowercase_keys: whether to convert every key to lower case.
        ignore_missing_paths: whether to ignore failures from missing files/folders.
@@ -70,6 +73,9 @@ def config(
         if isinstance(config_, Mapping):
             instances.append(config_from_dict(config_, **default_kwargs))
             continue
+        elif isinstance(config_, Configuration):
+            instances.append(config_)
+            continue
         elif isinstance(config_, str):
             if config_.endswith(".py"):
                 config_ = ("python", config_, *default_args)
@@ -81,8 +87,8 @@ def config(
                 config_ = ("toml", config_, True)
             elif config_.endswith(".ini"):
                 config_ = ("ini", config_, True)
-            elif config_.endswith(".env"):
-                config_ = ("dotenv", config_, True)
+            elif config_.endswith(".env") or config_.startswith(".env"):
+                config_ = ("dotenv", config_, True, *default_args)
             elif os.path.isdir(config_):
                 config_ = ("path", config_, remove_level)
             elif config_ in ("env", "environment"):
@@ -102,7 +108,13 @@ def config(
             instances.append(config_from_dict(*config_[1:], **default_kwargs))
         elif type_ in ("env", "environment"):
             params = list(config_[1:]) + default_args[(len(config_) - 1) :]
-            instances.append(config_from_env(*params, **default_kwargs))
+            instances.append(
+                config_from_env(
+                    *params,
+                    **default_kwargs,
+                    strip_prefix=strip_prefix,
+                ),
+            )
         elif type_ == "python":
             if len(config_) < 2:
                 raise ValueError("No path specified for python module")
@@ -112,6 +124,7 @@ def config(
                     *params,
                     **default_kwargs,
                     ignore_missing_paths=ignore_missing_paths,
+                    strip_prefix=strip_prefix,
                 ),
             )
         elif type_ == "json":
@@ -136,6 +149,7 @@ def config(
                     *config_[1:],
                     **default_kwargs,
                     ignore_missing_paths=ignore_missing_paths,
+                    strip_prefix=strip_prefix,
                 ),
             )
         elif type_ == "ini":
@@ -144,6 +158,7 @@ def config(
                     *config_[1:],
                     **default_kwargs,
                     ignore_missing_paths=ignore_missing_paths,
+                    strip_prefix=strip_prefix,
                 ),
             )
         elif type_ == "dotenv":
@@ -152,6 +167,7 @@ def config(
                     *config_[1:],
                     **default_kwargs,
                     ignore_missing_paths=ignore_missing_paths,
+                    strip_prefix=strip_prefix,
                 ),
             )
         elif type_ == "path":
@@ -177,9 +193,10 @@ class EnvConfiguration(Configuration):
 
     def __init__(
         self,
-        prefix: str,
+        prefix: str = "",
         separator: str = "__",
         *,
+        strip_prefix: bool = True,
         lowercase_keys: bool = False,
         interpolate: InterpolateType = False,
         interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -188,9 +205,11 @@ class EnvConfiguration(Configuration):
 
         prefix: prefix to filter environment variables with
         separator: separator to replace by dots
+        strip_prefix: whether to include the prefix
         lowercase_keys: whether to convert every key to lower case.
         """
         self._prefix = prefix
+        self._strip_prefix = strip_prefix
         self._separator = separator
         super().__init__(
             {},
@@ -206,9 +225,12 @@ class EnvConfiguration(Configuration):
         for key, value in os.environ.items():
             if not key.startswith(self._prefix + self._separator):
                 continue
-            result[
-                key[len(self._prefix) :].replace(self._separator, ".").strip(".")
-            ] = value
+            if self._strip_prefix:
+                result[
+                    key[len(self._prefix) :].replace(self._separator, ".").strip(".")
+                ] = value
+            else:
+                result[key.replace(self._separator, ".").strip(".")] = value
         super().__init__(
             result,
             lowercase_keys=self._lowercase,
@@ -221,6 +243,7 @@ def config_from_env(
     prefix: str,
     separator: str = "__",
     *,
+    strip_prefix: bool = True,
     lowercase_keys: bool = False,
     interpolate: InterpolateType = False,
     interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -230,6 +253,7 @@ def config_from_env(
     Params:
         prefix: prefix to filter environment variables with.
         separator: separator to replace by dots.
+        strip_prefix: whether to include the prefix
         lowercase_keys: whether to convert every key to lower case.
         interpolate: whether to apply string interpolation when looking for items.
 
@@ -239,6 +263,7 @@ def config_from_env(
     return EnvConfiguration(
         prefix,
         separator,
+        strip_prefix=strip_prefix,
         lowercase_keys=lowercase_keys,
         interpolate=interpolate,
         interpolate_type=interpolate_type,
@@ -349,7 +374,7 @@ class FileConfiguration(Configuration):
 
     def __init__(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
         *,
         lowercase_keys: bool = False,
@@ -370,13 +395,15 @@ class FileConfiguration(Configuration):
             interpolate=interpolate,
             interpolate_type=interpolate_type,
         )
-        self._filename = data if read_from_file and isinstance(data, str) else None
+        self._filename = (
+            data if read_from_file and isinstance(data, (str, Path)) else None
+        )
         self._ignore_missing_paths = ignore_missing_paths
         self._reload_with_check(data, read_from_file)
 
     def _reload_with_check(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
     ) -> None:  # pragma: no cover
         try:
@@ -388,7 +415,7 @@ class FileConfiguration(Configuration):
 
     def _reload(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
     ) -> None:  # pragma: no cover
         raise NotImplementedError()
@@ -404,12 +431,12 @@ class JSONConfiguration(FileConfiguration):
 
     def _reload(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
     ) -> None:
         """Reload the JSON data."""
         if read_from_file:
-            if isinstance(data, str):
+            if isinstance(data, (str, Path)):
                 with open(data, "rt") as f:
                     result = json.load(f)
             else:
@@ -420,7 +447,7 @@ class JSONConfiguration(FileConfiguration):
 
 
 def config_from_json(
-    data: Union[str, TextIO],
+    data: Union[str, Path, TextIO],
     read_from_file: bool = False,
     *,
     lowercase_keys: bool = False,
@@ -456,10 +483,11 @@ class INIConfiguration(FileConfiguration):
 
     def __init__(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
         *,
         section_prefix: str = "",
+        strip_prefix: bool = True,
         lowercase_keys: bool = False,
         interpolate: InterpolateType = False,
         interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -467,6 +495,7 @@ class INIConfiguration(FileConfiguration):
     ):
         """Class Constructor."""
         self._section_prefix = section_prefix
+        self._strip_prefix = strip_prefix
         super().__init__(
             data=data,
             read_from_file=read_from_file,
@@ -476,7 +505,11 @@ class INIConfiguration(FileConfiguration):
             ignore_missing_paths=ignore_missing_paths,
         )
 
-    def _reload(self, data: Union[str, TextIO], read_from_file: bool = False) -> None:
+    def _reload(
+        self,
+        data: Union[str, Path, TextIO],
+        read_from_file: bool = False,
+    ) -> None:
         """Reload the INI data."""
         import configparser
 
@@ -487,7 +520,7 @@ class INIConfiguration(FileConfiguration):
                 return super().optionxform(optionstr) if lowercase else optionstr
 
         if read_from_file:
-            if isinstance(data, str):
+            if isinstance(data, (str, Path)):
                 with open(data, "rt") as f:
                     data = f.read()
             else:
@@ -495,8 +528,9 @@ class INIConfiguration(FileConfiguration):
         data = cast(str, data)
         cfg = ConfigParser()
         cfg.read_string(data)
+        n = len(self._section_prefix) if self._strip_prefix else 0
         result = {
-            section[len(self._section_prefix) :] + "." + k: v
+            section[n:] + "." + k: v
             for section, values in cfg.items()
             for k, v in values.items()
             if section.startswith(self._section_prefix)
@@ -505,10 +539,11 @@ class INIConfiguration(FileConfiguration):
 
 
 def config_from_ini(
-    data: Union[str, TextIO],
+    data: Union[str, Path, TextIO],
     read_from_file: bool = False,
     *,
     section_prefix: str = "",
+    strip_prefix: bool = True,
     lowercase_keys: bool = False,
     interpolate: InterpolateType = False,
     interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -531,6 +566,7 @@ def config_from_ini(
         data,
         read_from_file,
         section_prefix=section_prefix,
+        strip_prefix=strip_prefix,
         lowercase_keys=lowercase_keys,
         interpolate=interpolate,
         interpolate_type=interpolate_type,
@@ -543,11 +579,12 @@ class DotEnvConfiguration(FileConfiguration):
 
     def __init__(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
         prefix: str = "",
         separator: str = "__",
         *,
+        strip_prefix: bool = True,
         lowercase_keys: bool = False,
         interpolate: InterpolateType = False,
         interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -555,7 +592,9 @@ class DotEnvConfiguration(FileConfiguration):
     ):
         """Class Constructor."""
         self._prefix = prefix
+        self._strip_prefix = strip_prefix
         self._separator = separator
+
         super().__init__(
             data=data,
             read_from_file=read_from_file,
@@ -567,25 +606,24 @@ class DotEnvConfiguration(FileConfiguration):
 
     def _reload(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
     ) -> None:
         """Reload the .env data."""
         if read_from_file:
-            if isinstance(data, str):
+            if isinstance(data, (str, Path)):
                 with open(data, "rt") as f:
                     data = f.read()
             else:
                 data = data.read()
         data = cast(str, data)
         result: Dict[str, Any] = dict(
-            (y.strip() for y in x.split("=", 1))  # type: ignore
-            for x in data.splitlines()
-            if x
+            parse_env_line(x) for x in data.splitlines() if x and not x.startswith("#")
         )
 
+        n = len(self._prefix) if self._strip_prefix else 0
         result = {
-            k[len(self._prefix) :].replace(self._separator, ".").strip("."): v
+            k[n:].replace(self._separator, ".").strip("."): v
             for k, v in result.items()
             if k.startswith(self._prefix)
         }
@@ -594,11 +632,12 @@ class DotEnvConfiguration(FileConfiguration):
 
 
 def config_from_dotenv(
-    data: Union[str, TextIO],
+    data: Union[str, Path, TextIO],
     read_from_file: bool = False,
     prefix: str = "",
     separator: str = "__",
     *,
+    strip_prefix: bool = True,
     lowercase_keys: bool = False,
     interpolate: InterpolateType = False,
     interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -606,6 +645,8 @@ def config_from_dotenv(
 ) -> Configuration:
     """Create a [Configuration][config.configuration.Configuration] instance from a .env type file.
 
+    Lines starting with a # are ignored and treated as comments.
+
     Params:
         data: path to a .env type file or contents.
         read_from_file: whether to read from a file path or to interpret.
@@ -622,6 +663,7 @@ def config_from_dotenv(
         read_from_file,
         prefix=prefix,
         separator=separator,
+        strip_prefix=strip_prefix,
         lowercase_keys=lowercase_keys,
         interpolate=interpolate,
         interpolate_type=interpolate_type,
@@ -634,10 +676,11 @@ class PythonConfiguration(Configuration):
 
     def __init__(
         self,
-        module: Union[str, ModuleType],
+        module: Union[str, Path, ModuleType],
         prefix: str = "",
         separator: str = "_",
         *,
+        strip_prefix: bool = True,
         lowercase_keys: bool = False,
         interpolate: InterpolateType = False,
         interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -651,7 +694,8 @@ class PythonConfiguration(Configuration):
         lowercase_keys: whether to convert every key to lower case.
         """
         try:
-            if isinstance(module, str):
+            if isinstance(module, (str, Path)):
+                module = str(module)
                 if module.endswith(".py"):
                     import importlib.util
                     from importlib import machinery
@@ -669,6 +713,7 @@ class PythonConfiguration(Configuration):
                     module = importlib.import_module(module)
             self._module: Optional[ModuleType] = module
             self._prefix = prefix
+            self._strip_prefix = strip_prefix
             self._separator = separator
         except (FileNotFoundError, ModuleNotFoundError):
             if not ignore_missing_paths:
@@ -691,10 +736,9 @@ class PythonConfiguration(Configuration):
                 for x in dir(self._module)
                 if not x.startswith("__") and x.startswith(self._prefix)
             ]
+            n = len(self._prefix) if self._strip_prefix else 0
             result = {
-                k[len(self._prefix) :]
-                .replace(self._separator, ".")
-                .strip("."): getattr(self._module, k)
+                k[n:].replace(self._separator, ".").strip("."): getattr(self._module, k)
                 for k in variables
             }
         else:
@@ -708,10 +752,11 @@ class PythonConfiguration(Configuration):
 
 
 def config_from_python(
-    module: Union[str, ModuleType],
+    module: Union[str, Path, ModuleType],
     prefix: str = "",
     separator: str = "_",
     *,
+    strip_prefix: bool = True,
     lowercase_keys: bool = False,
     interpolate: InterpolateType = False,
     interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -733,6 +778,7 @@ def config_from_python(
         module,
         prefix,
         separator,
+        strip_prefix=strip_prefix,
         lowercase_keys=lowercase_keys,
         interpolate=interpolate,
         interpolate_type=interpolate_type,
@@ -796,7 +842,7 @@ class YAMLConfiguration(FileConfiguration):
 
     def __init__(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
         *,
         lowercase_keys: bool = False,
@@ -818,9 +864,13 @@ class YAMLConfiguration(FileConfiguration):
             ignore_missing_paths=ignore_missing_paths,
         )
 
-    def _reload(self, data: Union[str, TextIO], read_from_file: bool = False) -> None:
+    def _reload(
+        self,
+        data: Union[str, Path, TextIO],
+        read_from_file: bool = False,
+    ) -> None:
         """Reload the YAML data."""
-        if read_from_file and isinstance(data, str):
+        if read_from_file and isinstance(data, (str, Path)):
             with open(data, "rt") as f:
                 loaded = yaml.load(f, Loader=yaml.FullLoader)
         else:
@@ -831,7 +881,7 @@ class YAMLConfiguration(FileConfiguration):
 
 
 def config_from_yaml(
-    data: Union[str, TextIO],
+    data: Union[str, Path, TextIO],
     read_from_file: bool = False,
     *,
     lowercase_keys: bool = False,
@@ -866,10 +916,11 @@ class TOMLConfiguration(FileConfiguration):
 
     def __init__(
         self,
-        data: Union[str, TextIO],
+        data: Union[str, Path, TextIO],
         read_from_file: bool = False,
         *,
         section_prefix: str = "",
+        strip_prefix: bool = True,
         lowercase_keys: bool = False,
         interpolate: InterpolateType = False,
         interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -882,6 +933,7 @@ class TOMLConfiguration(FileConfiguration):
             )
 
         self._section_prefix = section_prefix
+        self._strip_prefix = strip_prefix
         super().__init__(
             data=data,
             read_from_file=read_from_file,
@@ -891,10 +943,14 @@ class TOMLConfiguration(FileConfiguration):
             ignore_missing_paths=ignore_missing_paths,
         )
 
-    def _reload(self, data: Union[str, TextIO], read_from_file: bool = False) -> None:
+    def _reload(
+        self,
+        data: Union[str, Path, TextIO],
+        read_from_file: bool = False,
+    ) -> None:
         """Reload the TOML data."""
         if read_from_file:
-            if isinstance(data, str):
+            if isinstance(data, (str, Path)):
                 with open(data, "rb") as f:
                     loaded = toml.load(f)
             else:
@@ -904,8 +960,9 @@ class TOMLConfiguration(FileConfiguration):
             loaded = toml.loads(data)
         loaded = cast(dict, loaded)
 
+        n = len(self._section_prefix) if self._section_prefix else 0
         result = {
-            k[len(self._section_prefix) :]: v
+            k[n:]: v
             for k, v in self._flatten_dict(loaded).items()
             if k.startswith(self._section_prefix)
         }
@@ -914,10 +971,11 @@ class TOMLConfiguration(FileConfiguration):
 
 
 def config_from_toml(
-    data: Union[str, TextIO],
+    data: Union[str, Path, TextIO],
     read_from_file: bool = False,
     *,
     section_prefix: str = "",
+    strip_prefix: bool = True,
     lowercase_keys: bool = False,
     interpolate: InterpolateType = False,
     interpolate_type: InterpolateEnumType = InterpolateEnumType.STANDARD,
@@ -939,6 +997,7 @@ def config_from_toml(
         data,
         read_from_file,
         section_prefix=section_prefix,
+        strip_prefix=strip_prefix,
         lowercase_keys=lowercase_keys,
         interpolate=interpolate,
         interpolate_type=interpolate_type,
